

#include <Stepper.h>

#define PIN_BUZZER 5
#define PIN_FINECORSA 9

#define PIN_TOUCH 7

#define MAX_POSITION 220
#define ALIGNMENT 10
#define TIME_CONF 6000

#define TIME_SHOOT 2000

Stepper s = Stepper(20, 10,11,12,13);     

int pos = 0;
int snum = 0;

void setup() 
{
  Serial.begin(9600);
  InitIO();  //program IO line
  alignFineCorsa(); //align cursor to zero position, using fine-corsa as reference
}

void loop() 
{
  readStepConfig(); //wait for # of touch + sleep sequence
  goToZero(); //go to zero again
  
  waitForStart(); //wait for a touch
  
  runProcedure(); //perform step and sleep procedure till the end
  
  goToZero(); //go to zero again
} 

//----------- SUPPORT FUNCTIONS -----------

void readStepConfig()
{
  unsigned long prev;
  boolean prevtouch = false;

  s.setSpeed(100);

  beep(1100,100,50);
  beep(1100,100,50);
  beep(1100,100,50);

  snum = 0;
  
  while(!(snum > 0 && ((millis() - prev) > TIME_CONF))) //while not configured (snum > 0) && TIME_CONF gone since last touch
  {
    boolean touch = readTouch();

    if (touch && !prevtouch) //fronte di salita del touch
    {    
      moveMotor(1);
      snum += 1;
      delay(50);
    }
    
    if (touch)
      prev = millis();
    
    prevtouch = touch;
  }
  beep(1100,1000,0);
}

void waitForStart()
{
  delay (TIME_SHOOT);
 
  while(!readTouch()) //fronte di salita del touch
     ;
 
  while(readTouch()) //fronte di discesa del touch
      ;

  beep(1100,200,100);
  beep(1100,200,100);
  beep(1100,200,100);
  beep(1100,200,100);
  beep(1100,1000,100);

}

void runProcedure()
{
  s.setSpeed(100);

  delay(TIME_SHOOT);

  beep(1100, 100,TIME_SHOOT);
  while(moveMotor(snum))
  {
    beep(1100, 100,TIME_SHOOT);
  }
}

void InitIO()
{
  pinMode(PIN_FINECORSA, INPUT);
  pinMode(PIN_TOUCH, INPUT);
}

int moveMotor(int steps)
{
  if (steps > 0)
    steps = min(steps, MAX_POSITION - pos); //limita il numero di step fattibili per non andare oltre MAX_POSITION
  else if (steps < 0)
    steps = max(steps, 0 - pos); //limita il numero di step fattibili per non andare oltre ZERO
  
  if (steps)
  {
    s.step(steps);
    pos += steps;
  }
  
  return (steps) ;
}

void alignFineCorsa()
{
  s.setSpeed(100);
  
  if (readFineCorsa())
  {
    while (readFineCorsa())
        s.step(1);
    s.step(10);
  }
  
  while (!readFineCorsa())
    s.step(-1);
    
  s.step(ALIGNMENT);
  pos = 0;
}

void goToZero()
{
  s.setSpeed(100);
  moveMotor(-pos);
}

boolean readFineCorsa()
{
  return !digitalRead(PIN_FINECORSA);   
}

boolean readTouch()
{
  boolean touch = !digitalRead(PIN_TOUCH);
   Serial.print (touch ? "true" : "false");
   Serial.print ("\r\n");
   return touch;   
}

void beep(int freq, int duration, int sleep)
{
  tone(PIN_BUZZER, freq, duration);
  delay(duration);
  delay(sleep);
}
